SystemOrganization addCategory: #ConfigurationOfFuelPlatform!

Object subclass: #ConfigurationOfFuelPlatform
	instanceVariableNames: 'project customProjectAttributes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ConfigurationOfFuelPlatform'!

!ConfigurationOfFuelPlatform commentStamp: 'MaxLeske 11/20/2016 16:32' prior: 0!
We use a separate configuration to make it simpler to manage the platform classes. Using a separate configuration also makes it easy to load all the packages at once when we need to update them.!

!ConfigurationOfFuelPlatform class methodsFor: 'private' stamp: 'MaxLeske 11/20/2016 15:22'!
baseConfigurationClassIfAbsent: aBlock

	^Smalltalk
		at: #MetacelloBaseConfiguration
		ifAbsent: [ 
			self ensureMetacelloBaseConfiguration.
			Smalltalk at: #MetacelloBaseConfiguration ifAbsent: aBlock ].
! !

!ConfigurationOfFuelPlatform class methodsFor: 'private' stamp: 'MaxLeske 11/20/2016 15:22'!
bootstrapMetacelloFrom: repositoryUrl
  "Corresponds to version 1.0.0-beta.32.6"

  "KEEP MetacelloConfigTemplate class>>ensureMetacelloBaseConfiguration in synch!!"

  | platformPkg |
  self ensureGoferVersion: 'Gofer-Core-lr.115' repositoryUrl: repositoryUrl.
  #('Metacello-Core-dkh.678' 'Metacello-MC-dkh.674')
    do: [ :pkg | self bootstrapPackage: pkg from: repositoryUrl ].
  platformPkg := Smalltalk
    at: #'SystemVersion'
    ifPresent: [ :cl | 
      | versionString |
      versionString := cl current version.
      (versionString beginsWith: 'Squeak')
        ifTrue: [ 
          (versionString beginsWith: 'Squeak3')
            ifTrue: [ 'Metacello-Platform.squeak-dkh.5' ]
            ifFalse: [ 'Metacello-Platform.squeak-dkh.22' ] ]
        ifFalse: [ 
          (versionString beginsWith: 'Pharo')
            ifTrue: [ 
              self bootstrapPackage: 'Metacello-PharoCommonPlatform-dkh.2' from: repositoryUrl.
              (versionString beginsWith: 'Pharo2')
                ifTrue: [ 'Metacello-Platform.pharo20-dkh.33' ]
                ifFalse: [ 'Metacello-Platform.pharo-dkh.34' ] ] ] ].
  self bootstrapPackage: platformPkg from: repositoryUrl! !

!ConfigurationOfFuelPlatform class methodsFor: 'private' stamp: 'MaxLeske 11/20/2016 15:22'!
bootstrapPackage: aString from: aPath
    | repository version |
    repository := (MCCacheRepository default includesVersionNamed: aString)
        ifTrue: [ MCCacheRepository default ]
        ifFalse: [ MCHttpRepository location: aPath user: '' password: '' ].
    self
        retry: [ 
            repository
                versionReaderForFileNamed: aString , '.mcz'
                do: [ :reader | 
                    version := reader version.
                    version load.
                    version workingCopy repositoryGroup addRepository: repository ] ]! !

!ConfigurationOfFuelPlatform class methodsFor: 'private' stamp: 'MaxLeske 11/20/2016 15:22'!
ensureGoferVersion: goferVersion repositoryUrl: repositoryUrl
    "load the p=file goferVersion if Gofer isn't loaded or an earlier version of Gofer is currently loaded"

    | goferVersionNumber wc pName |
    (Smalltalk at: #'Gofer' ifAbsent: [  ]) == nil
        ifTrue: [ ^ self bootstrapPackage: goferVersion from: repositoryUrl ].
    goferVersionNumber := (goferVersion copyAfterLast: $.) asNumber.
    wc := [ ((Smalltalk at: #'GoferPackageReference') name: 'Gofer') workingCopy ]
        on: Error
        do: [ :ex | ex return: ((Smalltalk at: #'GoferPackageReference') name: 'Gofer-Core') workingCopy ].
    pName := wc ancestry ancestors first name.
    (pName copyAfterLast: $.) asNumber <= goferVersionNumber
        ifTrue: [ self bootstrapPackage: goferVersion from: repositoryUrl ]! !

!ConfigurationOfFuelPlatform class methodsFor: 'private' stamp: 'MaxLeske 11/20/2016 15:22'!
ensureMetacello

	(self baseConfigurationClassIfAbsent: []) ensureMetacello! !

!ConfigurationOfFuelPlatform class methodsFor: 'private' stamp: 'MaxLeske 11/20/2016 15:22'!
ensureMetacello: loadList
  "Bootstrap Metacello, retry using alternate repository, if primary repository is not accessible"

  Smalltalk
    at: #'MetacelloProject'
    ifAbsent: [ 
      | version error gofer |
      (Array
        with: 'http://smalltalkhub.com/mc/dkh/metacello/main'
        with: 'http://seaside.gemtalksystems.com/ss/metacello')
        do: [ :repositoryUrl | 
          "bootstrap Metacello"
          [ 
          self bootstrapMetacelloFrom: repositoryUrl.
          Smalltalk
            at: #'ConfigurationOfMetacello'
            ifAbsent: [ 
              self
                retry: [ 
                  gofer := (Smalltalk at: #'Gofer') new.
                  gofer
                    perform: #'url:' with: repositoryUrl;
                    perform: #'package:' with: 'ConfigurationOfMetacello';
                    perform: #'load' ] ].
          version := (Smalltalk at: #'ConfigurationOfMetacello') project
            version: #'previewBootstrap'.
          version load: loadList.
          self
            retry: [ 
              gofer := (Smalltalk at: #'Gofer') new.
              Smalltalk
                at: #'ConfigurationOfMetacelloPreview'
                ifAbsent: [ 
                  gofer
                    perform: #'url:' with: repositoryUrl;
                    perform: #'package:'
                      with: 'ConfigurationOfMetacelloPreview';
                    perform: #'load' ] ].
          version := (Smalltalk at: #'ConfigurationOfMetacelloPreview') project
            version: #'stable'.	"load latest from GitHub"
          version load: loadList.
          ^ self ]
            on: Error
            do: [ :ex | 
              error := ex.
              Transcript
                cr;
                show: 'failed ensureMetacello using ';
                show: repositoryUrl printString;
                show: ' : ';
                show: ex description printString;
                show: '...retrying'.	"try again"
              ex return: nil ] ].	"shouldn't get here unless the load failed ... throw an error"
      self
        error:
          'retry with alternate repository failed: ' , error description printString ]! !

!ConfigurationOfFuelPlatform class methodsFor: 'private' stamp: 'MaxLeske 11/20/2016 15:22'!
ensureMetacelloBaseConfiguration

	Smalltalk
		at: #MetacelloBaseConfiguration
		ifAbsent: [ 
			| repository version |
			repository := MCHttpRepository location: 'http://seaside.gemtalksystems.com/ss/metacello/' user: '' password: ''.
			repository
				versionReaderForFileNamed: 'Metacello-Base-DaleHenrichs.2.mcz'
				do: [ :reader | 
					version := reader version.
					version load.
					version workingCopy repositoryGroup addRepository: repository ] ]! !

!ConfigurationOfFuelPlatform class methodsFor: 'loading' stamp: 'MaxLeske 11/21/2016 20:47'!
loadAll
	"Load the #stable version defined for this platform. The #stable version is the version that is recommended to be used on this platform."

	"self load"

	<apiDocumentation>
	| config |
	config := self new
		loadAll;
		yourself.
	^ config project load: #default! !

!ConfigurationOfFuelPlatform class methodsFor: 'accessing' stamp: 'MaxLeske 11/20/2016 15:22'!
project

	^ self new project! !

!ConfigurationOfFuelPlatform class methodsFor: 'private' stamp: 'MaxLeske 11/20/2016 15:22'!
retry: aBlock
    self retry: aBlock retryCount: 3! !

!ConfigurationOfFuelPlatform class methodsFor: 'private' stamp: 'MaxLeske 11/20/2016 15:22'!
retry: aBlock retryCount: retryCount
    | count |
    count := 1.
    [ true ]
        whileTrue: [ 
            [ 
            aBlock value.
            ^ self ]
                on: Error
                do: [ :ex | 
                    count < retryCount
                        ifTrue: [ 
                            Transcript
                                cr;
                                show: 'RETRYING AFTER:';
                                cr;
                                show: ex description printString.
                            (Delay forSeconds: 5) wait ]
                        ifFalse: [ 
                            Transcript
                                cr;
                                show: 'FAILED RETRYING:';
                                cr;
                                show: ex description printString.
                            ex pass ] ].
            count := count + 1 ]! !

!ConfigurationOfFuelPlatform class methodsFor: 'unloading Metacello' stamp: 'MaxLeske 11/20/2016 15:22'!
unloadMetacello
	"Unload the classes that implement Metacello. Metacello is not needed once a project has been loaded, so it can safely be unloaded."

	"self unloadMetacello"

	<apiDocumentation>
	| gofer |
	gofer := (Smalltalk at: #Gofer) new.
	MCWorkingCopy allManagers do: [:wc |
		((wc packageName beginsWith: 'Metacello') or: [ wc packageName beginsWith: 'OB-Metacello' ])
			ifTrue: [ gofer package: wc packageName ]].
	gofer unload.! !

!ConfigurationOfFuelPlatform class methodsFor: 'development support' stamp: 'MaxLeske 11/20/2016 16:22'!
validate
	"Check the configuration for Errors, Critical Warnings, and Warnings (see class comment for MetacelloMCVersionValidator for more information). 
	Errors identify specification issues that will result in unexpected behaviour when you load the configuration. 
	Critical Warnings identify specification issues that may result in unexpected behavior when you load the configuration.
	Warnings identify specification issues that are technically correct, but are worth take a look at."

	"self validate"

	<apiDocumentation>
	self ensureMetacello.
	^ ((Smalltalk at: #MetacelloToolBox) validateConfiguration: self debug: #() recurse: false) explore! !

!ConfigurationOfFuelPlatform methodsFor: 'accessing' stamp: 'MaxLeske 11/20/2016 15:28'!
addCustomProjectAttribute: aSymbol
	customProjectAttributes := self customProjectAttributes copyWith: aSymbol! !

!ConfigurationOfFuelPlatform methodsFor: 'private' stamp: 'MaxLeske 11/20/2016 15:21'!
addPostloadHacks
	"Simply perform #addHacks again. We've now loaded classes that weren't
	there before and some of those also need extensions" 
	(Smalltalk at: #FLPlatform) current addHacks! !

!ConfigurationOfFuelPlatform methodsFor: 'baselines' stamp: 'MaxLeske 11/21/2016 20:34'!
baseline: spec
	<version: #'baseline'>
	spec for: #common do: [		
		spec
			blessing: #baseline;
			postLoadDoIt: #addPostloadHacks;
			package: 'FuelPlatform-Core'.
		self fuelRepository: spec.
			
		spec group: 'default' with: #('FuelPlatform-Core' 'FuelPlatformCurrentGroup'). ].
	
	self 
		pharoPackagesAndGroups: spec;
		squeakPackagesAndGroups: spec! !

!ConfigurationOfFuelPlatform methodsFor: 'accessing' stamp: 'MaxLeske 11/20/2016 15:27'!
customProjectAttributes
	^ customProjectAttributes ifNil: [ customProjectAttributes := self platformAttributes ]! !

!ConfigurationOfFuelPlatform methodsFor: 'symbolic versions' stamp: 'MaxLeske 11/20/2016 16:15'!
default: spec
	<symbolicVersion: #default>
	spec for: #common version: 'baseline'! !

!ConfigurationOfFuelPlatform methodsFor: 'helpers' stamp: 'MaxLeske 11/20/2016 15:19'!
fuelRepository: spec 
	spec repository: 'http://smalltalkhub.com/mc/Pharo/Fuel/main'! !

!ConfigurationOfFuelPlatform methodsFor: 'accessing' stamp: 'smalltalkCI 7/2/2017 10:16'!
loadAll
	#(pharo 'pharo1.x' 'pharo2.x' 'pharo3.x' 'pharo4.x' 'pharo5.x' 'pharo6.x' 'pharo7.x'
		squeak 'squeak4.x' 'squeak5.x' 'squeak6.x') do: [ :attribute |
			self addCustomProjectAttribute: attribute ]! !

!ConfigurationOfFuelPlatform methodsFor: 'helpers' stamp: 'JurajKubelka 5/29/2019 08:51'!
pharoPackagesAndGroups: spec
	spec for: #pharo do: [
		spec package: 'FuelPlatform-Pharo-Core' with: [ spec requires: 'FuelPlatform-Core' ] ].
	spec for: #'pharo1.x' do: [
		spec package: 'FuelPlatform-Pharo-01x' with: [ spec requires: 'FuelPlatform-Pharo-Core' ].
		spec group: 'FuelPlatformCurrentGroup' with: 'FuelPlatform-Pharo-01x' ].
	spec for: #'pharo2.x' do: [
		spec package: 'FuelPlatform-Pharo-02' with: [ spec requires: 'FuelPlatform-Pharo-Core' ].
		spec group: 'FuelPlatformCurrentGroup' with: 'FuelPlatform-Pharo-02' ].
	spec for: #'pharo3.x' do: [
		spec package: 'FuelPlatform-Pharo-03' with: [ spec requires: 'FuelPlatform-Pharo-Core' ].
		spec group: 'FuelPlatformCurrentGroup' with: 'FuelPlatform-Pharo-03' ].
	spec for: #'pharo4.x' do: [
		spec package: 'FuelPlatform-Pharo-04' with: [ spec requires: 'FuelPlatform-Pharo-Core' ].
		spec group: 'FuelPlatformCurrentGroup' with: 'FuelPlatform-Pharo-04' ].
	spec for: #'pharo5.x' do: [
		spec package: 'FuelPlatform-Pharo-05' with: [ spec requires: 'FuelPlatform-Pharo-Core' ].
		spec group: 'FuelPlatformCurrentGroup' with: 'FuelPlatform-Pharo-05' ].
	spec for: #'pharo6.x' do: [
		spec package: 'FuelPlatform-Pharo-06' with: [ spec requires: 'FuelPlatform-Pharo-Core' ].
		spec group: 'FuelPlatformCurrentGroup' with: 'FuelPlatform-Pharo-06' ].
	spec for: #'pharo7.x' do: [
		spec package: 'FuelPlatform-Pharo-07' with: [ spec requires: 'FuelPlatform-Pharo-Core' ].
		spec group: 'FuelPlatformCurrentGroup' with: 'FuelPlatform-Pharo-07' ].
	spec for: #'pharo8.x' do: [
		spec package: 'FuelPlatform-Pharo-07' with: [ spec requires: 'FuelPlatform-Pharo-Core' ].
		spec group: 'FuelPlatformCurrentGroup' with: 'FuelPlatform-Pharo-07' ].! !

!ConfigurationOfFuelPlatform methodsFor: 'private' stamp: 'JurajKubelka 5/29/2019 08:47'!
platformAttributes
	(SystemVersion current version beginsWith: 'Pharo8') ifTrue: [ ^ #('pharo8.x') ].
	(SystemVersion current version beginsWith: 'Pharo7') ifTrue: [ ^ #('pharo7.x') ].
	(SystemVersion current version beginsWith: 'Pharo6') ifTrue: [ ^ #('pharo6.x') ].
	(SystemVersion current version beginsWith: 'Pharo5') ifTrue: [ ^ #('pharo5.x') ].
	(SystemVersion current version beginsWith: 'Pharo4') ifTrue: [ ^ #('pharo4.x') ].
	(SystemVersion current version beginsWith: 'Pharo3') ifTrue: [ ^ #('pharo3.x') ].
	(SystemVersion current version beginsWith: 'Pharo2') ifTrue: [ ^ #('pharo2.x') ].
	(SystemVersion current version beginsWith: 'Pharo6') ifTrue: [ ^ #('pharo6.x') ].
	(SystemVersion current version beginsWith: 'Pharo1.') ifTrue: [ ^ #('pharo1.x') ].
	(#('Pharo1.1' 'Pharo-1.1') anySatisfy: [ :version |
			SystemVersion current version beginsWith: version ]) ifTrue: [ ^ #('pharo1.x') ].
	
	(SystemVersion current version beginsWith: 'Squeak6') ifTrue: [ ^ #('squeak6.x') ].
	(SystemVersion current version beginsWith: 'Squeak5') ifTrue: [ ^ #('squeak5.x') ].
	((SystemVersion current version beginsWith: 'Squeak4.6') or: [
			"See #fixSystemVersion"
			[ (Smalltalk imageName includesSubString: '4.6') ]
				on: MessageNotUnderstood
				do: [ false ] ]) ifTrue: [ ^ #('squeak4.x') ].
	(SystemVersion current version beginsWith: 'Squeak4.') ifTrue: [ ^ #('squeak4.x') ].
	
	Error signal: 'Could not determine image version'! !

!ConfigurationOfFuelPlatform methodsFor: 'accessing' stamp: 'MaxLeske 11/20/2016 15:24'!
project
    ^ project
        ifNil: [ 
            "Bootstrap Metacello if it is not already loaded"
            self class ensureMetacello.
            project := self projectClass new projectAttributes: self customProjectAttributes.	"Create the Metacello project"
            (Smalltalk at: #'MetacelloVersionConstructor') on: self project: project. "Construct the project"
            project loadType: #'linear'.	"change to #atomic if desired"
            project ]! !

!ConfigurationOfFuelPlatform methodsFor: 'accessing' stamp: 'MaxLeske 11/20/2016 15:24'!
project: aProject

	project ifNil: [ self class ensureMetacello ].
	project := aProject! !

!ConfigurationOfFuelPlatform methodsFor: 'accessing' stamp: 'MaxLeske 11/20/2016 15:24'!
projectClass
    ^ Smalltalk at: #MetacelloMCProject! !

!ConfigurationOfFuelPlatform methodsFor: 'helpers' stamp: 'MaxLeske 3/19/2017 20:54'!
squeakPackagesAndGroups: spec
	spec for: #squeak do: [
		spec package: 'FuelPlatform-Squeak-Core' with: [ spec requires: 'FuelPlatform-Core' ] ].
	spec for: #'squeak4.x' do: [
		spec package: 'FuelPlatform-Squeak-04x' with: [ spec requires: 'FuelPlatform-Squeak-Core' ].
		spec group: 'FuelPlatformCurrentGroup' with: 'FuelPlatform-Squeak-04x' ].
	spec for: #'squeak5.x' do: [
		spec package: 'FuelPlatform-Squeak-05' with: [ spec requires: 'FuelPlatform-Squeak-Core' ].
		spec group: 'FuelPlatformCurrentGroup' with: 'FuelPlatform-Squeak-05' ].
	spec for: #'squeak6.x' do: [
		spec package: 'FuelPlatform-Squeak-06' with: [ spec requires: 'FuelPlatform-Squeak-Core' ].
		spec group: 'FuelPlatformCurrentGroup' with: 'FuelPlatform-Squeak-06' ]! !
